/* tableInfo.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "tableInfo.h"
#include "tableDef.h"

Database *music_db = NULL;

ostream &operator << (ostream &out, TableID table_id) {
  switch (table_id) {
  case TI_songs:
    return out << "songs";
  case TI_songs_extension:
    return out << "songs_extension";
  case TI_cds:
    return out << "cds";
  case TI_cds_extension:
    return out << "cds_extension";
  case TI_artists:
    return out << "artists";
  case TI_artists_extension:
    return out << "artists_extension";
  case TI_configure:
    return out << "configure";
  case TI_volumes:
    return out << "volumes";
  case TI_soundfile:
    return out << "soundfile";
  case TI_combo_items:
    return out << "combo_items";
  }

  return out << "Invalid(" << (int)table_id << ")";
}

// Presently, all tables are in the music database.
Database *get_table_db(TableID table_id) {
  return music_db;
}

string
get_table_name(TableID table_id) {
  switch (table_id) {
  case TI_songs:
    return "songs";
  case TI_songs_extension:
    return "songs_extension";
  case TI_cds:
    return "cds";
  case TI_cds_extension:
    return "cds_extension";
  case TI_artists:
    return "artists";
  case TI_artists_extension:
    return "artists_extension";
  case TI_configure:
    return "configure";
  case TI_volumes:
    return "volumes";
  case TI_soundfile:
    return "soundfile";
  case TI_combo_items:
    return "combo_items";
  }

  cerr << "Unexpected table id in get_table_name(): " << table_id << "\n";
  abort();

  // This is just to make the compiler happy.  We'll never get here.
  return string();
}

string
get_table_key(TableID table_id) {
  switch (table_id) {
  case TI_songs:
  case TI_songs_extension:
    return "song_id";

  case TI_cds:
  case TI_cds_extension:
    return "cd_id";

  case TI_artists:
  case TI_artists_extension:
    return "artist_id";

  case TI_configure:
    return "";

  case TI_volumes:
    return "volume_id";

  case TI_soundfile:
    return "soundfile_id";

  case TI_combo_items:
    return "combo_item_id";
  }

  cerr << "Unexpected table id in get_table_key(): " << table_id << "\n";
  abort();

  // This is just to make the compiler happy.  We'll never get here.
  return string();
}

string
get_table_key_sequence(TableID table_id) {
  assert(!is_extension_table(table_id));

  return get_table_name(table_id) + "_" + get_table_key(table_id) + "_seq";
}

bool 
is_extension_table(TableID table_id) {
  switch (table_id) {
  case TI_songs:
  case TI_cds:
  case TI_artists:
  case TI_configure:
  case TI_volumes:
  case TI_soundfile:
  case TI_combo_items:
    return false;

  case TI_songs_extension:
  case TI_cds_extension:
  case TI_artists_extension:
    return true;
  }

  cerr << "Unexpected table id in is_extension_table(): " << table_id << "\n";
  abort();

  // This is just to make the compiler happy.  We'll never get here.
  return false;
}


TableDef *
get_songs_table() {
  static TableDef *def = NULL;

  if (def != NULL) {
    return def;
  }

  def = new TableDef;
  def->_table_id = TI_songs;

  def->define_pkey("song_id", "Song");

  def->define_field(SFI_performer, DataCell::DC_foreign_key, TI_artists, 
		    DataCell(), "performer", "Performer");
  def->define_field(SFI_writer, DataCell::DC_foreign_key, TI_artists,
		    DataCell(), "writer", "Writer");
  def->define_field(SFI_length, DataCell::DC_elapsed_time, TI_no_table,
		    DataCell(), "length", "Length");
  def->define_field(SFI_loaded_time, DataCell::DC_timestamp, TI_no_table,
		    DataCell(), "loaded_time", "Date loaded");
  def->define_field(SFI_file_type, DataCell::DC_foreign_key, TI_soundfile,
		    DataCell(), "file_type", "File type");
  def->define_field(SFI_volume_id, DataCell::DC_foreign_key, TI_volumes,
		    DataCell(), "volume_id", "Volume");
  def->define_field(SFI_name, DataCell::DC_string, TI_no_table,
		    DataCell(), "name", "Name");
  def->define_field(SFI_filename, DataCell::DC_string, TI_no_table,
		    DataCell(), "filename", "Filename");

  return def;
}

TableDef *
get_cds_table() {
  static TableDef *def = NULL;

  if (def != NULL) {
    return def;
  }

  def = new TableDef;
  def->_table_id = TI_cds;

  def->define_pkey("cd_id", "CD");

  def->define_field(CFI_cddb, DataCell::DC_string, TI_no_table,
		    DataCell(), "cddb", "CDDB disc ID");
  def->define_field(CFI_performer, DataCell::DC_foreign_key, TI_artists,
		    DataCell(), "performer", "Performer");
  def->define_field(CFI_writer, DataCell::DC_foreign_key, TI_artists,
		    DataCell(), "writer", "Writer");
  def->define_field(CFI_title, DataCell::DC_string, TI_no_table,
		    DataCell(), "title", "Title");

  return def;
}

TableDef *
get_artists_table() {
  static TableDef *def = NULL;

  if (def != NULL) {
    return def;
  }

  def = new TableDef;
  def->_table_id = TI_artists;

  // We need to have default values for the last_name, first_name, and
  // group_prefix of the empty string.  These values must never be
  // null, because we need to sort based on the concatenation of these
  // three values.

  def->define_pkey("artist_id", "Artist");

  def->define_field(AFI_last_name, DataCell::DC_string, TI_no_table,
		    DataCell(""), "last_name", "Last Name");
  def->define_field(AFI_first_name, DataCell::DC_string, TI_no_table,
		    DataCell(""), "first_name", "First Name");
  def->define_field(AFI_name_suffix, DataCell::DC_string, TI_no_table,
		    DataCell(), "name_suffix", "Jr., Sr., etc.");
  def->define_field(AFI_group_prefix, DataCell::DC_string, TI_no_table,
		    DataCell(""), "group_prefix", "A, An, The");
  def->define_field(AFI_group_name, DataCell::DC_string, TI_no_table,
		    DataCell(), "group_name", "Group Name");

  return def;
}

TableDef *
get_configure_table() {
  static TableDef *def = NULL;

  if (def != NULL) {
    return def;
  }

  def = new TableDef;
  def->_table_id = TI_configure;

  def->define_field(CFI_database_version, DataCell::DC_float, TI_no_table,
		    DataCell(), "database_version", "Database Version");
  def->define_field(CFI_cdrom_type, DataCell::DC_foreign_key, TI_soundfile,
		    DataCell(), "cdrom_type", "CD-Rom file type");
  def->define_field(CFI_cdrom_load_type, DataCell::DC_foreign_key, TI_soundfile,
		    DataCell(), "cdrom_load_type", "Load from CD as type");
  def->define_field(CFI_cdrom_want_type, DataCell::DC_foreign_key, TI_soundfile,
		    DataCell(), "cdrom_want_type", "Encode from CD to type");
  def->define_field(CFI_cdrom_load_volume, DataCell::DC_foreign_key, TI_volumes,
		    DataCell(), "cdrom_load_volume", "Load from CD into volume");
  def->define_field(CFI_cdrom_want_volume, DataCell::DC_foreign_key, TI_volumes,
		    DataCell(), "cdrom_want_volume", "Encode from CD into volume");

  return def;
}

TableDef *
get_volumes_table() {
  static TableDef *def = NULL;

  if (def != NULL) {
    return def;
  }

  def = new TableDef;
  def->_table_id = TI_volumes;

  def->define_field(VFI_bucket_size, DataCell::DC_int, TI_no_table,
		    DataCell(), "bucket_size", "Bucket Size (MB)");
  def->define_field(VFI_current_bucket, DataCell::DC_int, TI_no_table,
		    DataCell(), "current_bucket", "Current Bucket");
  def->define_field(VFI_description, DataCell::DC_string, TI_no_table,
		    DataCell(), "description", "Description");
  def->define_field(VFI_definition, DataCell::DC_string, TI_no_table,
		    DataCell(), "definition", "Definition");
  def->define_field(VFI_bucket_name, DataCell::DC_string, TI_no_table,
		    DataCell(), "bucket_name", "Bucket directory name");

  return def;
}

TableDef *
get_soundfile_table() {
  static TableDef *def = NULL;

  if (def != NULL) {
    return def;
  }

  def = new TableDef;
  def->_table_id = TI_soundfile;

  def->define_field(SFFI_sajber_backend, DataCell::DC_bool, TI_no_table,
		    DataCell(), "sajber_backend", "Sajber backend");
  def->define_field(SFFI_description, DataCell::DC_string, TI_no_table,
		    DataCell(), "description", "Description");
  def->define_field(SFFI_play_command, DataCell::DC_string, TI_no_table,
		    DataCell(), "play_command", "Play Command");
  def->define_field(SFFI_play_response, DataCell::DC_string, TI_no_table,
		    DataCell(), "play_response", "Play Response");
  def->define_field(SFFI_verify_command, DataCell::DC_string, TI_no_table,
		    DataCell(), "verify_command", "Verify Command");
  def->define_field(SFFI_verify_response, DataCell::DC_string, TI_no_table,
		    DataCell(), "verify_response", "Verify Response");

  return def;
}

TableDef *
get_combo_items_table() {
  static TableDef *def = NULL;

  if (def != NULL) {
    return def;
  }

  def = new TableDef;
  def->_table_id = TI_combo_items;

  def->define_pkey("combo_item_id", "Combo Item");

  def->define_field(CIFI_label, DataCell::DC_string, TI_no_table,
		    DataCell(), "label", "Item Label");

  return def;
}

TableDef *
get_system_table(TableID table_id) {
  assert(!is_extension_table(table_id));

  switch (table_id) {
  case TI_songs:
    return get_songs_table();
  case TI_cds:
    return get_cds_table();
  case TI_artists:
    return get_artists_table();
  case TI_configure:
    return get_configure_table();
  case TI_volumes:
    return get_volumes_table();
  case TI_soundfile:
    return get_soundfile_table();
  case TI_combo_items:
    return get_combo_items_table();
  }

  cerr << "Unexpected table id in get_system_table(): " << table_id << "\n";
  abort();

  // This is just to make the compiler happy.  We'll never get here.
  return NULL;
}

void
define_all_tables() {
  TableDef::get_table(TI_songs);
  TableDef::get_table(TI_songs_extension);
  TableDef::get_table(TI_cds);
  TableDef::get_table(TI_cds_extension);
  TableDef::get_table(TI_artists);
  TableDef::get_table(TI_artists_extension);
  TableDef::get_table(TI_configure);
  TableDef::get_table(TI_volumes);
  TableDef::get_table(TI_soundfile);
  TableDef::get_table(TI_combo_items);
}
